﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic.Classification {
  [StorableClass]
  [Item("Generational Classification Sliding Window Analyzer", "")]
  public class GenerationalClassificationSlidingWindowAnalyzer : GenerationalSlidingWindowAnalyzer {
    private const string ModelCreatorParameterName = "ModelCreator";
    #region parameter properties
    public ILookupParameter<ISymbolicClassificationModelCreator> ModelCreatorParameter {
      get { return (ILookupParameter<ISymbolicClassificationModelCreator>)Parameters[ModelCreatorParameterName]; }
    }
    #endregion
    #region properties
    private ISymbolicClassificationModelCreator ModelCreator {
      get { return ModelCreatorParameter.ActualValue; }
      set { ModelCreatorParameter.ActualValue = value; }
    }
    #endregion

    [StorableConstructor]
    protected GenerationalClassificationSlidingWindowAnalyzer(bool deserializing) : base(deserializing) { }
    protected GenerationalClassificationSlidingWindowAnalyzer(GenerationalClassificationSlidingWindowAnalyzer original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new GenerationalClassificationSlidingWindowAnalyzer(this, cloner);
    }
    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
    }

    public GenerationalClassificationSlidingWindowAnalyzer() {
      Parameters.Add(new LookupParameter<ISymbolicClassificationModelCreator>(ModelCreatorParameterName));
    }

    public override IOperation Apply() {
      if (!ResultCollection.ContainsKey("Best Solutions")) {
        ResultCollection.Add(new Result("Best Solutions", new SlidingWindowBestClassificationSolutionsCollection()));
      }
      var bestSolutions = (SlidingWindowBestClassificationSolutionsCollection)ResultCollection["Best Solutions"].Value;
      bestSolutions.ModelCreator = ModelCreator;
      return base.Apply();
    }
  }
}
